import subprocess, random, os, glob, sys, signal
import numpy as np
import pandas as pd
import networkx as nx
from scipy.stats import pearsonr, lognorm
from collections import Counter, defaultdict

class TimeoutException(Exception):
   pass

def timeout_handler(signum, frame):
   raise TimeoutException

signal.signal(signal.SIGALRM, timeout_handler)

# Generate powerlaw popularity distribution for news sources
sys.stderr.write("Generating sources...\n")
os.mkdir("synth_data/run%s" % sys.argv[1])
os.chdir("synth_data/run%s" % sys.argv[1])

source_popularity = pd.read_csv("../data/popularity_distribution_fb.csv", header = None, names = ("popularity",))["popularity"].values.astype(int)

# Generate a normal polarity distribution with avg = 0, and min/max -1/1
source_truthfulness = lognorm.rvs(0.5, size = source_popularity.shape[0])
source_truthfulness = (source_truthfulness / np.max(np.abs(source_truthfulness)))
with open("source_truthfulness", 'w') as f:
   for i in range(len(source_truthfulness)):
      f.write("%s\t%s\n" % (i, source_truthfulness[i]))

# Generate the polarity of the audience, with avg = 0, and min/max -1/1
sys.stderr.write("Generating social & audience networks...\n")
number_users = int(max(source_popularity) * 2.5)
print("# users: %d" % number_users)

# Make the social network, an LFR benchmark
signal.alarm(60)
try:
   with open("time_seed.dat", 'w') as f:
      f.write("%s" % random.randint(0, 10000000))
   bash_command = "../../benchmark -N %s -k 9 -mu 0.1 -t1 3 -t2 1.1 -maxk 50" % number_users
   process = subprocess.Popen(bash_command.split(), stdout = subprocess.PIPE)
   output, error = process.communicate()
   signal.alarm(0)
except TimeoutException:
   sys.stderr.write("Social network creation fail!\n")
   sys.exit()

G = nx.read_edgelist("network.dat", nodetype = int)
nx.set_node_attributes(G, {n: 0 for n in G.nodes}, name = "polarity")
nx.write_edgelist(G, "social_network", data = False)
user_community = pd.read_csv("community.dat", sep = "\t", header = None, names = ("user", "community"))

for _ in glob.glob("*.dat"):
   os.remove(_)

# Assign users to news sources. Each news source gets its popularity worth of users,
# and we pick them so that they're the closest possible in polarity
G_source_user = nx.Graph()
user_nsources = defaultdict(int)
users = set(G.nodes)
for i in range(len(source_popularity)):
   audience = random.sample(users, source_popularity[i])
   for audience_member in audience:
      G_source_user.add_edge("s%s" % i, "u%s" % audience_member)
      user_nsources[audience_member] += 1

nx.write_edgelist(G_source_user, "audience_network", data = False)
with open("user_nsources", 'w') as f:
   for n in G.nodes:
      f.write("%s\t%s\n" % (n, user_nsources[n]))

print("avg sources per user: %1.2f" % (sum(user_nsources.values()) / len(user_nsources)))
os.chdir("../..")
