import sys, os, ipdb
import pandas as pd
import networkx as nx

sys.stderr.write("Setting up simulations...\n")
audience = nx.read_edgelist("synth_data/run%s/audience_network" % sys.argv[2])
social = nx.read_edgelist("synth_data/run%s/social_network" % sys.argv[2])
source_truth = pd.read_csv("synth_data/run%s/source_truthfulness" % sys.argv[2], sep = "\t", header = None, names = ("source", "truthfulness"))

social = nx.relabel_nodes(social, {n: "u%s" % n for n in social.nodes})
source_truth["source"] = source_truth["source"].map(lambda x: "s%s" % x)
source_truth = source_truth.set_index("source").to_dict()["truthfulness"]

shareability = float(sys.argv[1])

if not os.path.exists("flags%s/" % shareability):
   os.makedirs("flags%s/" % shareability)

i = 0
for flaggability in (.1, .2, .3, .4, .5, .6, .7, .8, .9):
   tot_cascade_size = 0
   truth_flags = []
   sys.stderr.write("Running simulations...\n")
   for publisher in source_truth:
      hit_audience = set(audience.neighbors(publisher))
      active_audience = hit_audience
      flags = 0
      while len(active_audience) > 0:
         new_audience = set()
         for user in active_audience:
            if source_truth[publisher] > (1 - shareability):
               new_audience |= ((set(social.neighbors(user)) - hit_audience) - active_audience)
            elif source_truth[publisher] < (1 - flaggability):
               flags += 1
         hit_audience |= new_audience
         active_audience = new_audience
      tot_cascade_size += len(hit_audience)
      if flags > 0:
         truth_flags.append((flags, source_truth[publisher]))
      i += 1
      if i % 1024 == 0:
         sys.stderr.write("%1.0f%%\n" % (100 * i / (8146 * 9)))
   truth_flags = pd.DataFrame(truth_flags, columns = ("flag", "truthfulness"))
   truth_flags.to_csv("flags%s/truth_flags_%s_%s.csv" % (shareability, flaggability, sys.argv[2]), sep = "\t", index = False)
   print("Total cascade size: %s" % tot_cascade_size)
