import sys, os
import numpy as np
import pandas as pd
import networkx as nx

sys.stderr.write("Setting up simulations...\n")
audience = nx.read_edgelist("synth_data/run%s/audience_network" % sys.argv[2])
social = nx.read_edgelist("synth_data/run%s/social_network" % sys.argv[2])
users = pd.read_csv("synth_data/run%s/user_polarity" % sys.argv[2], sep = "\t", header = None, names = ("user", "polarity"))
source_polarity = pd.read_csv("synth_data/run%s/source_polarity" % sys.argv[2], sep = "\t", header = None, names = ("source", "polarity"))
source_truth = pd.read_csv("synth_data/run%s/source_truthfulness" % sys.argv[2], sep = "\t", header = None, names = ("source", "truthfulness"))

social = nx.relabel_nodes(social, {n: "u%s" % n for n in social.nodes})
users["user"] = users["user"].map(lambda x: "u%s" % x)
userlist = list(users["user"])
source_polarity["source"] = source_polarity["source"].map(lambda x: "s%s" % x)
source_truth["source"] = source_truth["source"].map(lambda x: "s%s" % x)
users = users.set_index("user").to_dict()["polarity"]
source_polarity = source_polarity.set_index("source").to_dict()["polarity"]
source_truth = source_truth.set_index("source").to_dict()["truthfulness"]

shareability = float(sys.argv[1])

if not os.path.exists("flags%s/" % shareability):
   os.makedirs("flags%s/" % shareability)

i = 0
for flaggability in (.1, .2, .3, .4, .5, .6, .7, .8, .9):
   flaggabilities = np.random.normal(loc = flaggability, scale = flaggability / 8, size = len(users))
   shareabilities = flaggabilities * (shareability / flaggability)
   flaggabilities = {userlist[i]: flaggabilities[i] for i in range(len(userlist))}
   shareabilities = {userlist[i]: shareabilities[i] for i in range(len(userlist))}
   tot_cascade_size = 0
   polarity_flags = []
   sys.stderr.write("Running simulations...\n")
   for publisher in source_polarity:
      hit_audience = set(audience.neighbors(publisher))
      active_audience = hit_audience
      flags = 0
      while len(active_audience) > 0:
         new_audience = set()
         for user in active_audience:
            polarity_delta = abs(users[user] - source_polarity[publisher])
            flag_factor = source_truth[publisher] / polarity_delta
            flag_factor = 1 - (flag_factor / (flag_factor + 1)) #TODO: changed
            if flag_factor < shareabilities[user]:
               new_audience |= ((set(social.neighbors(user)) - hit_audience) - active_audience)
            elif flag_factor > flaggabilities[user]:
               flags += 1
         hit_audience |= new_audience
         active_audience = new_audience
      tot_cascade_size += len(hit_audience)
      if flags > 0:
         polarity_flags.append((source_polarity[publisher], flags, source_truth[publisher]))
      i += 1
      if i % 1024 == 0:
         sys.stderr.write("%1.0f%%\n" % (100 * i / (8146 * 9)))
   polarity_flags = pd.DataFrame(polarity_flags, columns = ("polarity", "flag", "truthfulness"))
   polarity_flags.to_csv("flags%s/polarity_flags_%s_%s.csv" % (shareability, flaggability, sys.argv[2]), sep = "\t", index = False)
   print("Total cascade size: %s" % tot_cascade_size)
